@@ -48,39 +48,12 @@ def test_mnemosyne10():
 def test_supermemo_xml_01_unicode():
     deck = DeckStorage.Deck()
     deck.addModel(BasicModel())
-    file = unicode(os.path.join(testDir, "importing/supermemo_ENGLISHFORBEGGINERS_noOEM.xml"))
+    file = unicode(os.path.join(testDir, "importing/supermemo1.xml"))
     i = supermemo_xml.SupermemoXmlImporter(deck, file)
     #i.META.logToStdOutput = True
     i.doImport()
-    assert i.total == 92
-    deck.s.close()
-
-def test_supermemo_xml_02_escaped():
-    deck = DeckStorage.Deck()
-    deck.addModel(BasicModel())
-    file = unicode(os.path.join(testDir, "importing/supermemo_ENGLISHFORBEGGINERS_oem_1250.xml"))
-    i = supermemo_xml.SupermemoXmlImporter(deck, file)
-    i.doImport()
-    assert i.total == 30
-    deck.s.close()
-
-def test_supermemo_xml_03():
-    deck = DeckStorage.Deck()
-    deck.addModel(BasicModel())
-    file = unicode(os.path.join(testDir, "importing/supermemo_EnglishPronunciationTop100.xml"))
-    i = supermemo_xml.SupermemoXmlImporter(deck, file)
-    #i.META.logToStdOutput = True
-    i.doImport()
-    assert i.total == 100
-    deck.s.close()
-
-def test_supermemo_xml_04():
-    deck = DeckStorage.Deck()
-    deck.addModel(BasicModel())
-    file = unicode(os.path.join(testDir, "importing/supermemo_ENGLISHVOCABULARYBUILDER.xml"))
-    i = supermemo_xml.SupermemoXmlImporter(deck, file)
-    i.doImport()
-    assert i.total == 60
+    # only returning top-level elements?
+    assert i.total == 1
     deck.s.close()
 
 def test_anki10():
@@ -100,7 +73,7 @@ def test_anki10():
     deck.close()
     # import a deck into itself - 10-2 is the same as test10, but with one
     # card answered and another deleted. nothing should be synced to client
-    deck = DeckStorage.Deck(file)
+    deck = DeckStorage.Deck(file, backup=False)
     i = anki10.Anki10Importer(deck, file2)
     i.doImport()
     assert i.total == 0
